import numpy as np

progress_indicator = in_stream["Progress Indicator"]
in_data_stream = in_stream["Data Stream"]
start = progress_indicator.getIntOffsetIndex()
total_size = in_data_stream.getSize()

if start >= total_size:
  return

chunk_size = 1024 * 1024
end = min(start + chunk_size, total_size)

log_data = ''.join(in_data_stream.readChunkList(start, end))

# Some lines may contain several contiguous JSON objects
# Normalize to one JSON object per line
newlines_offset = log_data.count('}{')
log_data = log_data.replace('}{', '}\n{')

# Last chunk may be an incomplete JSON object: leave it for next iteration
log_data_lines = log_data.splitlines()
if len(log_data_lines) < 2:
  return
log_data = '\n'.join(log_data_lines[:-1])

end = start + len(log_data) - newlines_offset + 1

log_data = log_data.decode('utf8')

e_rab_data_array = None
e_utran_data_array = None

for array in out_array:
  if array['variation'] == 'e_rab':
    e_rab_data_array = array['Data Array']
  if array['variation'] == 'e_utran':
    e_utran_data_array = array['Data Array']

try:
  vt, vInititialEPSBEstabSR, vAddedEPSBEstabSR, evt, vIPThp_qci = context.Base_getORSKPIValue(log_data)
except AssertionError:
  # If some data is invalid: ignore data chunk and move on rather than crashing the data analysis activity
  progress_indicator.setIntOffsetIndex(end)
  return

e_rab_dtype = np.dtype([
  ('vt', 'float'),
  ('vInititialEPSBEstabSR_lo', 'float64'),
  ('vInititialEPSBEstabSR_hi', 'float64'),
  ('vAddedEPSBEstabSR_lo', 'float64'),
  ('vAddedEPSBEstabSR_hi', 'float64'),
])

e_utran_dtype = np.dtype([
  ('evt', 'float'),
  ('dl_lo', 'float64'),
  ('dl_hi', 'float64'),
  ('ul_lo', 'float64'),
  ('ul_hi', 'float64'),
])

e_rab_array = e_rab_data_array.getArray()
if not e_rab_array:
  e_rab_array = e_rab_data_array.initArray(shape=(0,), dtype=e_rab_dtype)
e_rab_array_data = []

e_utran_array = e_utran_data_array.getArray()
if not e_utran_array:
  e_utran_array = e_utran_data_array.initArray(shape=(0,), dtype=e_utran_dtype )
e_utran_array_data = []

for i in range(len(vt)):
  e_rab_array_data.append((vt[i], vInititialEPSBEstabSR[i][0], vInititialEPSBEstabSR[i][1], vAddedEPSBEstabSR[i][0], vAddedEPSBEstabSR[i][1]))

for i in range(len(evt)):
  for j in vIPThp_qci[i]:
    e_utran_array_data.append((evt[i], j[0], j[1], j[2], j[3]))

if e_rab_array_data:
  e_rab_array_data =  np.ndarray((len(e_rab_array_data), ), e_rab_dtype, np.array(e_rab_array_data))
  e_rab_array.append(e_rab_array_data)
if e_utran_array_data:
  e_utran_array_data =  np.ndarray((len(e_utran_array_data), ), e_utran_dtype, np.array(e_utran_array_data))
  e_utran_array.append(e_utran_array_data)

progress_indicator.setIntOffsetIndex(end)

return
